/*
** Filename: HIDUart.c
**
** Automatically created by Application Wizard 1.4.2
**
** Part of solution HIDUart in project HIDUart
**
** Comments:
**
** Important: Sections between markers "FTDI:S*" and "FTDI:E*" will be overwritten by
** the Application Wizard
*/

#include "HIDUart.h"

/* Default settings for UART interface */
#define DEF_UART_BAUD		UART_BAUD_9600
#define DEF_UART_DATA_BITS	UART_DATA_BITS_8
#define DEF_UART_FLOW		UART_FLOW_RTS_CTS
#define DEF_UART_STOP_BITS	UART_STOP_BITS_1
#define DEF_UART_PARITY		UART_PARITY_NONE

/* FTDI:STP Thread Prototypes */
vos_tcb_t *tcbFIRMWARE;

void firmware();
/* FTDI:ETP */

/* FTDI:SDH Driver Handles */
VOS_HANDLE hUSBHOST_1; // USB Host Port 1
VOS_HANDLE hUART; // UART Interface Driver
/* FTDI:EDH */

/* Declaration for IOMUx setup function */
void iomux_setup(void);

/* Main code - entry point to firmware */
void main(void)
{
	/* FTDI:SDD Driver Declarations */
	// UART Driver configuration context
	uart_context_t uartContext;
	// USB Host configuration context
	usbhost_context_t usbhostContext;
	/* FTDI:EDD */

	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(512);
	/* FTDI:EKI */

	iomux_setup();

	/* FTDI:SDI Driver Initialisation */
	// Initialise UART
	uartContext.buffer_size = VOS_BUFFER_SIZE_128_BYTES;
	uart_init(VOS_DEV_UART,&uartContext);

	// Initialise USB Host
	usbhostContext.if_count = 8;
	usbhostContext.ep_count = 16;
	usbhostContext.xfer_count = 2;
	usbhostContext.iso_xfer_count = 2;
	usbhost_init(VOS_DEV_USBHOST_1, -1, &usbhostContext);
	/* FTDI:EDI */

	/* FTDI:SCT Thread Creation */
	tcbFIRMWARE = vos_create_thread_ex(20, 4096, firmware, "Application", 0);
	/* FTDI:ECT */

	vos_start_scheduler();

main_loop:
	goto main_loop;
}

/* FTDI:SSP Support Functions */

unsigned char usbhost_connect_state(VOS_HANDLE hUSB)
{
	unsigned char connectstate = PORT_STATE_DISCONNECTED;
	usbhost_ioctl_cb_t hc_iocb;

	if (hUSB)
	{
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_GET_CONNECT_STATE;
		hc_iocb.get        = &connectstate;
		vos_dev_ioctl(hUSB, &hc_iocb);

		// repeat if connected to see if we move to enumerated
		if (connectstate == PORT_STATE_CONNECTED)
		{
			vos_dev_ioctl(hUSB, &hc_iocb);
		}
	}
	return connectstate;
}

/* FTDI:ESP */

void open_drivers(void)
{
	/* Code for opening and closing drivers - move to required places in Application Threads */
	/* FTDI:SDA Driver Open */
	hUSBHOST_1 = vos_dev_open(VOS_DEV_USBHOST_1);
	hUART = vos_dev_open(VOS_DEV_UART);
	/* FTDI:EDA */
}

void attach_drivers(void)
{
	/* FTDI:SUA Layered Driver Attach Function Calls */
	/* FTDI:EUA */
}

void close_drivers(void)
{
	/* FTDI:SDB Driver Close */
	vos_dev_close(hUSBHOST_1);
	vos_dev_close(hUART);
	/* FTDI:EDB */
}

/* Application Threads */
void firmware(void)
{
	// receive buffer
	char *buf;
	unsigned char status;

	// device handle
	usbhost_device_handle_ex ifDev;
	// endpoint handles
	usbhost_ep_handle_ex epInt, epCtrl;

	// completion semaphore
	vos_semaphore_t semRead;

	// Host Controller ioctl request block
	usbhost_ioctl_cb_t hc_iocb;
	usbhost_ioctl_cb_class_t hc_ioctlClass;

	// interrupt endpoint transfer descriptor
	usbhost_xfer_t xfer;

	// UART ioctl request block
	common_ioctl_cb_t uart_iocb;

	// host controller device descriptor
	usb_deviceRequest_t desc_dev;

	// endpoint information descriptor
	usbhost_ioctl_cb_ep_info_t epInfo;

	// GPIO and LEDs
	unsigned char leds;

	vos_init_semaphore(&semRead, 0);

	open_drivers();

	uart_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
	uart_iocb.set.param = DMA_ACQUIRE_AS_REQUIRED;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set baud rate
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
	uart_iocb.set.uart_baud_rate = DEF_UART_BAUD;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set flow control
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
	uart_iocb.set.param = DEF_UART_FLOW;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set data bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
	uart_iocb.set.param = DEF_UART_DATA_BITS;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set stop bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
	uart_iocb.set.param = DEF_UART_STOP_BITS;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set parity
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
	uart_iocb.set.param = DEF_UART_PARITY;
	vos_dev_ioctl(hUART, &uart_iocb);

	// Host Controller Setup
	hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_SET_HANDLE_MODE_EXTENDED;
	vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

	// GPIO Set LED pins to outputs
	vos_gpio_set_port_mode(GPIO_PORT_A, 0xff);  // set all as output

	do
	{
		// wait for enumeration to complete
		vos_delay_msecs(250);
		leds = LED3;
		vos_gpio_write_port(GPIO_PORT_A, leds);
		vos_delay_msecs(250);
		leds = 0;
		vos_gpio_write_port(GPIO_PORT_A, leds);

		// find first HID on USB Port 1
		hc_ioctlClass.dev_class = USB_CLASS_HID;
		hc_ioctlClass.dev_subclass = USB_SUBCLASS_ANY;
		hc_ioctlClass.dev_protocol = USB_PROTOCOL_ANY;

		// user ioctl to find first hub device
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_FIND_HANDLE_BY_CLASS;
		hc_iocb.handle.dif = NULL;
		hc_iocb.set = &hc_ioctlClass;
		hc_iocb.get = &ifDev;

		status = vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

		if (status != USBHOST_OK)
		{
			continue;
		}

		leds = LED4;
		vos_gpio_write_port(GPIO_PORT_A, leds);

		// user ioctl to find interrupt endpoint on this device
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_GET_INT_IN_ENDPOINT_HANDLE;
		hc_iocb.handle.dif = ifDev;
		hc_iocb.get = &epInt;

		status = vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

		if (status != USBHOST_OK)
		{
			continue;
		}

		// user ioctl to find interrupt endpoint on this device
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_GET_CONTROL_ENDPOINT_HANDLE;
		hc_iocb.handle.dif = ifDev;
		hc_iocb.get = &epCtrl;

		status = vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

		if (status != USBHOST_OK)
		{
			continue;
		}

		// user ioctl to find interrupt endpoint information
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_GET_ENDPOINT_INFO;
		hc_iocb.handle.ep = epInt;
		hc_iocb.get = &epInfo;

		status = vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

		if (status != USBHOST_OK)
		{
			continue;
		}

		// allocate buffer for data from HID device
		buf = (char *)malloc(epInfo.max_size);

		// send a SetIdle to the device
		desc_dev.bmRequestType = USB_BMREQUESTTYPE_HOST_TO_DEV |
			USB_BMREQUESTTYPE_CLASS |
			USB_BMREQUESTTYPE_INTERFACE;
		desc_dev.bRequest = 0x0a;
		desc_dev.wValue = 0;
		desc_dev.wIndex = 0;
		desc_dev.wLength = 0;

		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_SETUP_TRANSFER;
		hc_iocb.handle.ep = epCtrl;
		hc_iocb.set = &desc_dev;

		vos_dev_ioctl(hUSBHOST_1, &hc_iocb);

		while (1)
		{
			// prepare for reading data from device
			xfer.s = &semRead;
			xfer.ep = epInt;
			xfer.buf = buf;
			// max size of interrupt endpoint data
			xfer.len = epInfo.max_size;
			xfer.flags = 0;

			status = vos_dev_read(hUSBHOST_1, (unsigned char *) &xfer, sizeof(usbhost_xfer_t), NULL);

			if (status != USBHOST_OK)
			{
				leds = LED5;
				vos_gpio_write_port(GPIO_PORT_A, leds);
				break;
			}

			leds = 0;
			vos_gpio_write_port(GPIO_PORT_A, leds);

			vos_dev_write(hUART, (unsigned char *)buf, epInfo.max_size, NULL);				leds = LED4;

			leds = LED4;
			vos_gpio_write_port(GPIO_PORT_A, leds);
		}

		// release buffer for data from HID device
		free(buf);
	}
	while (1);
}

